#!/bin/ksh

######################################################################
######################################################################

report_function()
{
  case $1 in
    warn) if [[ -a "$logFile" ]]; then
            print "$PROGNAME warning:  $2, continuing ..." >> $logFile
          fi
          #$NLS-E$ "$PROGNAME warning:  $2, continuing ..."
          print -u2 "$PROGNAME `nls_replace 'KCI0535I'`:  $2, `nls_replace 'KCI0536I'` ..."
          sleep 2
          return
          ;;
    fail) if [[ -a "$logFile" ]]; then
            print "$PROGNAME failure:  $2."  >> $logFile
            #print "$PROGNAME failure:  $2."
            print -u2 "`nls_replace 'KCI1200E' "UpdateAutoRun.sh" $logFile`"
          else
            print "$PROGNAME failure:  $2."
          fi
          exit 2
          ;;
    info) if [[ -a "$logFile" ]]; then
            print "$PROGNAME info:  $2" >> $logFile
          else
            print "$PROGNAME info:  $2"
          fi
          return
          ;;
    infoCont) if [[ -a "$logFile" ]]; then
            print "$PROGblnk     :  $2" >> $logFile
          else
            print "$PROGblnk     :  $2"
          fi
          return
          ;;
  esac
}

Use_report() {
    print -u2 "\n$PROGNAME [ -h install_dir ] \n"
    exit 2
}

######################################################################
######################################################################

script_pream()
{
   print "#!/bin/ksh" > $1
   print "# (C) COPYRIGHT IBM, Inc. 2007" >> $1
   print "# Unpublished Work" >> $1
   print "# All Rights Reserved" >> $1
   print "# IBM/Tivoli Monitoring Version 6.2.0" >> $1
   return
}

script_pream_so()
{
   print "#!/bin/sh" > $1
   print "# (C) COPYRIGHT IBM, Inc. 2007" >> $1
   print "# Unpublished Work" >> $1
   print "# All Rights Reserved" >> $1
   print "# IBM/Tivoli Monitoring Version 6.2.0" >> $1   
   return
}


######################################################################
######################################################################

#This is to get instance from env file.
getInstance()
{
  pCode=$1
  instance=""
  isExist=""

# For Oracle and Sybase we will check <hostname>_<pp>_<instance>.cfg file
  if [[ "$pCode" = @(or|oy|mq) ]]; then
    host=$(hostname | cut -d. -f1)
    for rec in $(ls "$CANDLEHOME"/config/"$host"_"$pCode"_*.cfg  "$CANDLEHOME"/config/"$host"\.*_"$pCode"_*.cfg 2>/dev/null |grep -v kcirunas | grep -v kdyrunas)
    do
      rec=${rec##*/}  # Remove the directory path.
      rec=${rec%.*}   # Remove the ".cfg" extension.
      instance=${rec##*_$pCode_}  # instance -> myinstance
      [[ -z "$instance" ]] && continue
      [[ -f "$startFile" ]] && isExist=$(grepInStart "agent -o $instance start $pCode")
      [[ -z "$isExist" ]] && writeAgentInstanceToStartStop $user $pCode $instance
      isExist=""
    done
    if [[ "$pCode" != "mq" ]]; then
      return
    fi
  fi

# the instance is saved as platform for certain agents
  instanceFile=$(cat $CANDLEHOME/config/.ConfigData/k${pCode}env)
  instanceFile=$(print "$instanceFile" | grep -v ^default | grep -v ^$os | grep -v FIELDSEP)
  instanceFile=$(print "$instanceFile" | cut -d"|" -f1 | sort -u)
  # defect 73338 - use -p for um instances
  if [[ "$pCode" = "um" ]] then
    parm="-p"
  else
    parm="-o"
  fi  
  if [[ -n "$instanceFile" ]]; then
    for instance in $instanceFile
    do
      [[ -f "$startFile" ]] && isExist=$(grepInStart "agent $parm $instance start $pCode")
      [[ -z "$isExist" ]] && writeAgentInstanceToStartStop $user $pCode $instance
      isExist=""
    done
  else

# Now let's check for env files.
#Some instances are saved as variables in the env file
    instanceFile=$(cat $CANDLEHOME/config/.ConfigData/k${pCode}env)
    instanceFile=$(print "$instanceFile" | grep -v ^default | grep -w INSTANCE)
    for instanceLine in $instanceFile
    do
      instance=$(print "$instanceLine"| cut -d"|" -f3)
      [[ -z "$instance" ]] && continue
      [[ -f "$startFile" ]] && isExist=$(grepInStart "agent $parm $instance start $pCode")
      [[ -z "$isExist" ]] && writeAgentInstanceToStartStop $user $pCode $instance
      isExist=""
     done
  fi

}

# BEGIN MAIN PART OF APAR IY97904 - create a MODE string that is used for zLinux machines
checkSetCmdMode()
{
        pcCode=$1

        #reset cmdMode so its new each time
        cmdMode=""

        if [[ -z "$pcCode" ]]
        then
         report_function fail "The product code was not specified - not continuing with mode check"
        fi

        print "checkSetCmdMode: pcCode: $pcCode" >> $logFile
        print "checkSetCmdMode: cArch: $cArch" >> $logFile

        if [[ "$cArch" = ls3* ]]
        then

                print "checkSetCmdMode: cArch matches a zLinux platform - continue processing & checking" >> $logFile

                checkZModeShl=$CANDLEHOME/bin/checkZMode.sh
                [[ -x "$checkZModeShl" ]] || report_function fail "Cannot execute \"$checkZModeShl\""

                print "checkSetCmdMode: executing $checkZModeShl to get real and current mode" >> $logFile

                realMode=`$checkZModeShl | grep realMode | cut -d":" -f2`
                currentMode=`$checkZModeShl | grep currentMode | cut -d":" -f2`

                if [[ "$realMode" = "64" ]]
                then
                        print "checkSetCmdMode: the real mode is 64, check if its PCs we want" >> $logFile

                        if [[ "$pcCode" = @(ms|cq) ]]
                        then
                                print "checkSetCmdMode: the pc is one we want - updating the cmdMode flag" >> $logFile
                                cmdMode="s390"
                        else
                                print "checkSetCmdMode: the pc is not one we want = no need to update that cmdMode flag" >> $logFile
                        fi
                else
                        print "checkSetCmdMode: the real mode is 32 = no need to update that cmdMode flag" >> $logFile
                fi

        else
                print "checkSetCmdMode: cArch does not match a zLinux platform - skip processing & checking" >> $logFile
        fi
}
# END MAIN PART OF APAR IY97904 - create a MODE string that is used for zLinux machines



######################################################################
######################################################################

writeAgentInstanceToStartStop()
{
  #set -x
  checkSetCmdMode $pCode
  # defect 73338 - use -p for um instances
  if [[ "$pCode" = "um" ]] then
    parm="-p"
  else
    parm="-o"
  fi

  if [[ "$2" != "cj" ]]
  then
    comment=$(grep "^[$WHITESPACE]*#.* -c \"[$WHITESPACE]*$CANDLEHOME/bin/itmcmd agent $parm $3 start $2" $backFile$baknum 2> /dev/null)
    commentMode=""
    [[ -n "$comment" ]] && commentMode="#"

    # The current behavior for this line is the following:
    #  - correct the fully qualified su command as specified by the su_value
    #  - if the command was commented out by the user, leave it commented out but correct the fully qualified su command
    #  - does not maintain a modified USER - the config/kcirunas.cfg must be used instead
    print "$commentMode $su_value - $1 -c \"$cmdMode $CANDLEHOME/bin/itmcmd agent $parm $3 start $2 >/dev/null 2>&1\""  >> $startFile
  fi

  comment=$(grep "^[$WHITESPACE]*#.* -c \"[$WHITESPACE]*$CANDLEHOME/bin/itmcmd agent $parm $3 stop $2" $backFile$baknum 2> /dev/null)
  commentMode=""
  [[ -n "$comment" ]] && commentMode="#"

  # The current behavior for this line is the following:
  #  - correct the fully qualified su command as specified by the su_value
  #  - if the command was commented out by the user, leave it commented out but correct the fully qualified su command
  #  - does not maintain a modified USER - the config/kcirunas.cfg must be used instead
  print "$commentMode $su_value - $1 -c \"$cmdMode $CANDLEHOME/bin/itmcmd agent $parm $3 stop $2 >/dev/null 2>&1\""  >> $stopFile

}


writeAgentToStartStop()
{
  #set -x
  [[ "$2" = "iw" ]] && return
  
  checkSetCmdMode $pCode
  
  if [[ "$2" != "cj" ]]
  then
    comment=$(grep "^[$WHITESPACE]*#.* -c \"[$WHITESPACE]*$CANDLEHOME/bin/itmcmd agent start $2" $backFile$baknum 2> /dev/null)
    commentMode=""
    [[ -n "$comment" ]] && commentMode="#"

    # The current behavior for this line is the following:
    #  - correct the fully qualified su command as specified by the su_value
    #  - if the command was commented out by the user, leave it commented out but correct the fully qualified su command
    #  - does not maintain a modified USER - the config/kcirunas.cfg must be used instead
    print "$commentMode $su_value - $1 -c \"$cmdMode $CANDLEHOME/bin/itmcmd agent start $2 >/dev/null 2>&1\""  >> $startFile
  fi

  if [ "$2" = "kf" ]; then
    # handle kf stop force option
    commentMode=""
	
	# This will check with or without the -f option - then flag the comment - but regardless, we always get -f back
    commentA=$(grep "^[$WHITESPACE]*#.* -c \"[$WHITESPACE]*$CANDLEHOME/bin/itmcmd agent stop $2" $backFile$baknum 2> /dev/null)
    [[ -n "$commentA" ]] && commentMode="#"
    commentB=$(grep "^[$WHITESPACE]*#.* -c \"[$WHITESPACE]*$CANDLEHOME/bin/itmcmd agent -f stop $2" $backFile$baknum 2> /dev/null)
    [[ -n "$commentB" ]] && commentMode="#"

    # The current behavior for this line is the following:
    #  - correct the fully qualified su command as specified by the su_value
    #  - if the command was commented out by the user, leave it commented out but correct the fully qualified su command
    #  - does not maintain a modified USER - the config/kcirunas.cfg must be used instead

    # this adds the -f option - the code before made it clear that they wanted to add -f regardless
    print "$commentMode $su_value - $1 -c \"$cmdMode $CANDLEHOME/bin/itmcmd agent -f stop $2 >/dev/null 2>&1\""  >> $stopFile

  else
    # handle all other agents
	
    comment=$(grep "^[$WHITESPACE]*#.* -c \"[$WHITESPACE]*$CANDLEHOME/bin/itmcmd agent stop $2" $backFile$baknum 2> /dev/null)
    commentMode=""
    [[ -n "$comment" ]] && commentMode="#"

    # The current behavior for this line is the following:
    #  - correct the fully qualified su command as specified by the su_value
    #  - if the command was commented out by the user, leave it commented out but correct the fully qualified su command
    #  - does not maintain a modified USER - the config/kcirunas.cfg must be used instead
    print "$commentMode $su_value - $1 -c \"$cmdMode $CANDLEHOME/bin/itmcmd agent stop $2 >/dev/null 2>&1\""  >> $stopFile

  fi
}

writeServerToStartStop()
{
  #set -x
  checkSetCmdMode $pCode

  comment=$(grep "^[$WHITESPACE]*#.* -c \"[$WHITESPACE]*$CANDLEHOME/bin/itmcmd server start $2" $backFile$baknum 2> /dev/null)
  commentMode=""
  [[ -n "$comment" ]] && commentMode="#"

  # The current behavior for this line is the following:
  #  - correct the fully qualified su command as specified by the su_value
  #  - if the command was commented out by the user, leave it commented out but correct the fully qualified su command
  #  - does not maintain a modified USER - the config/kcirunas.cfg must be used instead
  print "$commentMode $su_value - $1 -c \"$cmdMode $CANDLEHOME/bin/itmcmd server start $2 >/dev/null 2>&1\""  >> $startFile

  comment=$(grep "^[$WHITESPACE]*#.* -c \"[$WHITESPACE]*$CANDLEHOME/bin/itmcmd server stop $2" $backFile$baknum 2> /dev/null)
  commentMode=""
  [[ -n "$comment" ]] && commentMode="#"

  # The current behavior for this line is the following:
  #  - correct the fully qualified su command as specified by the su_value
  #  - if the command was commented out by the user, leave it commented out but correct the fully qualified su command
  #  - does not maintain a modified USER - the config/kcirunas.cfg must be used instead
  print "$commentMode $su_value - $1 -c \"$cmdMode $CANDLEHOME/bin/itmcmd server stop $2 >/dev/null 2>&1\""  >> $stopFile

}

######################################################################
######################################################################

getValueFromLine()
{
  print $(print $line | cut -d">" -f2 | cut -d"<" -f1)
}

getToken1FromLine()
{
  print $(print $line | cut -d">" -f1 | cut -d"<" -f2)
}

######################################################################
######################################################################

grepInStart()
{
  print $(grep "$1" $startFile 2> /dev/null)
}

######################################################################
######################################################################

update_rcFile()
{
print "$editString" > $SEDscript
checkit=0
cp -f $rcFile ${rcFile}.new
checkit=$?
if [[ $checkit = 0 ]]; then
  sed -f $SEDscript ${rcFile}.new > $rcFile
  checkit=$?
  if [[ $checkit != 0 ]]; then
    cp -f ${rcFile}.new $rcFile
  fi
fi
rm -f ${rcFile}.new
rm -f $SEDscript
}

######################################################################
######################################################################
# Check for <hostname>_kdyrunas.cfg file. The <hostname> is a short name only.
# <agent>
#  <productCode>gb</productCode>
#  <instance>
#   <name>instancename</name>
#   <user>userid</user>
#  </instance>
# </agent>


checkForCfg()
{
  pCode=$1
  runas=$2

  tmp=0
  hostFlag=false
  prodFlag=false
  instanceFlag=false
  host=$(hostname | cut -d. -f1)
  cfgname=$CANDLEHOME/config/${host}_${runas}.cfg
  [[ "$runas" = kcirunas ]] && cfgname=$CANDLEHOME/config/${runas}.cfg

  # if file does not exist, return
  [[ ! -f "$cfgname" ]] && return
  username=""
  instance=""
  while read line
  do
    token1=$(getToken1FromLine)

    if [[ "$token1" = "productCode" ]]; then
        if [[ "$(getValueFromLine)" = "$pCode" ]]; then
          prodFlag=true
        else
          prodFlag=false
        fi
    fi

    if [[ "$prodFlag" = true ]]; then
      case "$token1" in
        /agent)
          prodFlag=false
          ;;
        !productCode)
          prodFlag=false
          ;;
        name)
          instance=$(getValueFromLine)
          ;;
        user)
          username=$(getValueFromLine)
          ;;
        /instance)
          [[ -z "$username" ]] && username=$user
          if [[ -z "$instance" ]]
          then
             isExist=""
             [[ -f "$startFile" ]] && isExist=$(grepInStart "agent start $pCode")
             [[ -z "$isExist" ]] && writeAgentToStartStop $username $pCode
          else
             isExist=""
             if [[ "$pCode" = "ms" ]]; then
               [[ -f "$startFile" ]] && isExist=$(grepInStart "server start $instance")
               [[ -z "$isExist" ]] && writeServerToStartStop $username $instance
             else
               # defect 73338 - use -p for um instances
               if [[ "$pCode" = "um" ]] then
                 parm="-p"
               else
                 parm="-o"
               fi  
               [[ -f "$startFile" ]] && isExist=$(grepInStart "agent $parm $instance start $pCode" | grep $username)
               [[ -z "$isExist" ]] && writeAgentInstanceToStartStop $username $pCode $instance
             fi
          fi
          username=""
          instance=""
          ;;
      esac
    fi   #prodFlag

  done < $cfgname
  return
}


######################################################################
######################################################################
#This is to make sure certain agents have instance included otherwise
#this is an error.
verifyAgents()
{
  # if file does not exist, return
  [[ ! -f "$startFile" ]] && return
  while read readLine
  do
    instOpt=$(print $readLine | cut -d " " -f7)
    instPC=$(print $readLine | cut -d " " -f10)
    [[ "$instOpt" != "-o" && "$instOpt" != "-p" ]] && instPC=$(print $readLine | cut -d " " -f8)

    if [[ "$instPC" = "$pCode" && "$instOpt" != "-o" && "$instOpt" != "-p" ]]; then
      prodLine=$(grep "^$pCode" $CANDLEHOME/registry/proddsc.tbl)  # get product descript
      prodLine=$(print $prodLine| cut -d "|" -f2)
      report_function  warn  "$prodLine ($pCode) required start/stop with instance"
    fi
  done < $startFile
}


######################################################################
######################################################################
#Create two files based on current install.
#The scipt will write to restart script.

create_start_stop_file()
{
  startFile=$CANDLEHOME/tmp/start
  stopFile=$CANDLEHOME/tmp/stop
  pcFile=$CANDLEHOME/tmp/pc
  userFile=$CANDLEHOME/tmp/user
  startLine=""

  [[ -a "$startFile" ]] && rm $startFile
  [[ -a "$stopFile" ]] && rm $stopFile
  [[ -a "$pcFile" ]] && rm $pcFile
  [[ -a "$userFile" ]] && rm -f $userFile

  $CANDLEHOME/bin/cinfo -d > $pcFile

  [[ -a "$userTmp" ]] && rm -f $userTmp
  pc=$(grep "^\"..\"" $pcFile | cut -c2,3 | sort -r )
  for pCode in $pc
  do
  # if there is more than one arch.
    arch=$(ls $CANDLEHOME |grep ^$os)
    for Arch in $arch
    do
      # here Arch is the arch, pCode is pc
      if [[ -d "$CANDLEHOME/$Arch/$pCode/bin" ]]; then
        user=$(ls -ld "$CANDLEHOME/$Arch/$pCode/bin" | awk '{print $3}')
        if [[ "$pCode" != "ui" ]] && [[ "$pCode" != "gs" ]]; then
          instance=""

          tmpuser=$user
          inst=""
          checkForCfg $pCode "kdyrunas"
          checkForCfg $pCode "kcirunas"

          [[ "$pCode" != "ms" ]] && getInstance $pCode
          instance=""

          # this is to make sure ud, or, oy, sa and tx agents already have instance setup.
          if [[ "$pCode" = @(sa|tx|qv|gb|ph|ud|or|oy) ]]; then
            verifyAgents
            continue
          fi

	  [[ -f $CANDLEHOME/config/${pCode}.ini ]] && result=`grep "INSTALL_AGENT_TYPE=MULTI-INSTANCE" $CANDLEHOME/config/${pCode}.ini`
          if [[ -n $result ]]
          then
            verifyAgents
            continue
          fi

          if [[ "$pCode" = "ms" ]]; then
            cms=`ls $CANDLEHOME/tables`
            for temsName in $cms
             do
               [[ "$temsName" = @(cicatrsq|PLACEHOLDER) ]] && continue
               temsLength=`print $temsName| wc -c`
               temsLength=$((temsLength-4))
               oldTems=`print $temsName | cut -c ${temsLength}-`
               [[ "$oldTems" = "_old" ]] && continue
               [[ ! -d "$CANDLEHOME/tables/$temsName/ATTRLIB" ]] && continue
               [[ -f "$startFile" ]] && isExist=$(grepInStart "server start $temsName")
               [[ -z "$isExist" ]] && writeServerToStartStop $user $temsName
             done  #end cms
           else  #this is not ms, so we can find agent start pp" or "agent -o instance start pp"
             # defect 73338 - make sure we preserve the default um entry
             if [[ "$pCode" = "um" ]]; then
               [[ -f "$startFile" ]] && isExist=$(grepInStart "agent start $pCode")
             else
               [[ -f "$startFile" ]] && isExist=$(grepInStart "gent start $pCode")
             fi
             [[ -z "$isExist" ]] && writeAgentToStartStop $user $pCode
           fi  #end ms
        fi  #end of ui
      else 
        if [[ -f "$CANDLEHOME/$Arch/$pCode/kkfstart.sh" ]]; then
          user=$(ls -ld "$CANDLEHOME/$Arch/$pCode/kkfstart.sh" | awk '{print $3}')
	  [[ -f "$startFile" ]] && isExist=$(grepInStart "start $pCode")
	  [[ -z "$isExist" ]] && writeAgentToStartStop $user $pCode
	fi
      fi  #end of bin

    done #end arch
  done  #end pc
}

######################################################################
######################################################################

delete_start_all()
{
   # looping is not required to delete lines using sed instead of ed
   if [[ -n "$(grep 'gent start all' $rcFile 2>/dev/null)" ]]; then
      editString="/gent start all/d"
      update_rcFile
      if [[ $checkit != 0 ]]; then
         report_function fail "Delete of agent start all record failed."
      else
         report_function info "Delete of agent start all record successful."
      fi
   else
     return
   fi
}

delete_stop_all()
{
   # looping is not required to delete lines using sed instead of ed
   if [[ -n "$(grep 'gent stop all' "$rcFile" 2>/dev/null)" ]]; then
      editString="/gent stop all/d"
      update_rcFile
      if [[ $checkit != 0 ]]; then
         report_function fail "Delete of agent stop all record failed."
      else
         report_function info "Delete of agent stop all record successful."
      fi
   else
      return
   fi
}

######################################################################
######################################################################
#Add one line to script

add_start_line()
{
   startPath=$(print $line | cut -d"\"" -f2)
   if [[ -z "$(grep  "${startPath}"  $rcFile)" ]]; then
      print "$line" > $SEDline
      editString="/start_all()/{
N
r $SEDline
}"
      update_rcFile
      rm -f $SEDline
      if [[ $checkit != 0 ]]; then
         report_function fail "Unable to add start agent line in $rcFile."
      else
         report_function info "Add of start agent line successful."
      fi
   else
      report_function info "Start agent line already exists in $rcFile."
   fi
}

######################################################################
######################################################################
#Add one line to script

add_stop_line()
{
   stopPath=$(print "$line" | cut -d"\"" -f2)
   if [[ -z "$(grep  "${stopPath}"  $rcFile)" ]]; then
      print "$line" > $SEDline
      editString="/stop_all()/{
N
r $SEDline
}"
      update_rcFile
      rm -f $SEDline
      if [[ $checkit != 0 ]]; then
         report_function fail "Unable to add stop agent line in $rcFile."
      else
         report_function info "Add of stop agent line successful."
      fi
   else
      report_function info "Stop agent line already exists in $rcFile."
   fi
}
######################################################################
######################################################################





######################################################################
#   MAIN   ###########################################################
######################################################################

typeset logFile
rcFile=""
firstTime=false
typeset filenum
filenum=1
baknum=1

PROGNAME=$(basename $0)
PROGblnk="$(print $PROGNAME | sed 's/./ /g')"

#defect 58729: to support the install.sh from ITM6.1, also need to support the "UpdateAutoRun.sh <ch>"
CANDLEHOME=$1
while getopts :h: opt
do case $opt in
     h) CANDLEHOME=$OPTARG
        CANDLEHOME=$(eval print $CANDLEHOME)  # Process ~ if it was entered.
        ;;
     ?) Use_report
        ;;
   esac
done

[[ -n "$CANDLEHOME" ]] || CANDLEHOME=$(cd $(dirname $0) ; dirname $(pwd) )
export CANDLEHOME

#There is a TAB char in the second spot - DON'T remove it
WHITESPACE=" 	"

NLS_REPLACE_DIR=$CANDLEHOME/bin
PATH=$PATH:$NLS_REPLACE_DIR
export PATH

[[ ! -d "$CANDLEHOME" ]] && report_function fail "$CANDLEHOME is not a valid installation."

if [[ $(print "$CANDLEHOME" | wc -w) -ne 1 ]]; then
  report_function fail "Blanks are not permitted in CANDLEHOME: $CANDLEHOME"
fi

[[ -z "$(whoami 2>/dev/null)" ]] && alias whoami='id | cut -d"(" -f2 | cut -d")" -f1'

# if no AutoStart and not root exit now.
   myID=$(id 2> /dev/null | cut -d"=" -f2 2> /dev/null | cut -d"(" -f1 2> /dev/null)
   [[ "$myID" = "0" ]] || report_function fail "You must be \"root\" to update the auto start script."

logFile=$CANDLEHOME/logs/UpdateAutoRun.log
[[ -f $logFile ]] && rm $logFile 2>/dev/null
touch $logFile

SEDscript=$CANDLEHOME/tmp/SEDscript
SEDline=$CANDLEHOME/tmp/SEDline

dynShl=$CANDLEHOME/bin/dynarch.shl
[[ -x "$dynShl" ]] || report_function fail "Cannot execute \"$dynShl\""

regPath=$CANDLEHOME/registry
archTbl=$regPath/archdsc.tbl  # archTbl is needed by dynarch.shl
[[ -r "$archTbl" ]] || report_function fail "Cannot read \"$archTbl\""

# Determine the machine's architecture.

. $dynShl $archTbl  # dynarch.shl will determine a value for cArch.
[[ -n "$cArch" ]] || report_function fail "Could not determine the machine's architecture"
os=$(print "$cArch" |cut -c1-2)

# find the rc file base on os type
if [[  ! -f "$CANDLEHOME/registry/AutoStart" ]]; then
   export CANDLEHOME
   export cArch
   $CANDLEHOME/bin/AutoRunAgents.sh
   firstTime=true
fi

filenum=$(cat $CANDLEHOME/registry/AutoStart)

[[ "$os" = li ]] && rcFile=/etc/init.d/ITMAgents$filenum
[[ "$os" = lx ]] && rcFile=/etc/init.d/ITMAgents$filenum
[[ "$os" = lp ]] && rcFile=/etc/init.d/ITMAgents$filenum
[[ "$os" = ls ]] && rcFile=/etc/init.d/ITMAgents$filenum
[[ "$os" = hp ]] && rcFile=/sbin/init.d/ITMAgents$filenum
[[ "$os" = so ]] && rcFile=/etc/init.d/ITMAgents$filenum
[[ "$os" = ai ]] && rcFile=/etc/rc.itm$filenum
[[ -n "$rcFile" ]] || report_function fail "$os platform is not supported."

# If script doesn't exist.
if [[  ! -f "$rcFile" ]]; then
   export CANDLEHOME
   export cArch
   $CANDLEHOME/bin/AutoRunAgents.sh
   firstTime=true
fi

# IY96372 start =============================================================================
# Set up what value of su we are going to use - we only want to do this if the platform is HP,
# since that is the only platform right now having a problem. If su is found under the paths
# we want to check, we use them.. otherwise not, which is no worse than right now

#set the default value of su now, and then just use it if its not overwritten

su_value=`which su` 2> /dev/null
[ -f "$su_value" ] || su_value="su"

if [ "$os" = hp ]
then
  # we have an HP platform, we want to check two diff files

  if [[ -f /bin/su ]]
  then
    su_value="/bin/su"
  fi

  if [[ -f /usr/bin/su ]]
  then
    su_value="/usr/bin/su"
  fi

fi

#now at this point, either su_value was found and set on any other platform (or even HP maybe),
#or on HP, the other files were found and set properly - either way, we now have a proper su value
# IY96372 stop ==============================================================================

backFile=$CANDLEHOME/tmp/ITMAgents$filenum.bak
[[ "$os" = ai ]] && backFile=$CANDLEHOME/tmp/rc.itm$filenum.bak

[[ -r "$rcFile" ]] || report_function fail "Can't read from file $rcFile."
[[ -w "$rcFile" ]] || report_function fail "Can't write to file $rcFile."

# if rc file exist and not one we just created, copy to $CANDLEHOME/tmp for user to review.
if [[ "$firstTime" = false ]]; then
  if [[ -f "$rcFile" ]]; then
    while [[ -f "$backFile$baknum" ]]
    do
      baknum=`expr $baknum + 1`
    done

    cp $rcFile $backFile$baknum  2> /dev/null
    report_function info  "File $rcFile was backed up to"
    report_function infoCont  "$backFile$baknum"
    report_function infoCont  "before updates.  Please review"
    report_function infoCont  "$backFile$baknum"
    report_function infoCont  "and make changes to $rcFile"
    report_function infoCont  "if necessary.  Please remove"
    report_function infoCont  "$backFile$baknum"
    report_function infoCont  "after the review is complete."
    report_function infoCont  "Caution: Custom changes will be removed"
    report_function infoCont  "when UpdateAutoRun.sh is executed again."
  fi

  [[ "$os" = ai ]] && rmitab rcitm$filenum
# No need to remove the file.  The script_pream will empty it anyway.
#  rm -f $rcFile 2> /dev/null

# now let's create a new script.
  if [[ "$os" = so ]]
  then
    script_pream_so "$rcFile"
  else
    script_pream "$rcFile"
  fi
  
  if [[ "$os" = ai ]]
  then
    print "# Added by install utility to start instance <#> of the" 	>> $rcFile
    print "# IBM/Tivoli Monitoring agents" 				>> $rcFile
    print "#" 								>> $rcFile
    print ""                                                            >> $rcFile
    print ""                                                            >> $rcFile
    print "# IMPORTANT NOTE: =========================================================" >> $rcFile
    print "# Only commenting out a start/stop command will be preserved in this file." >> $rcFile
    print "# Any other user modifications will not be preserved. User ID overrides" >> $rcFile
    print "# must be made in the config/kcirunas.cfg file to be preserved." >> $rcFile
    print "# IMPORTANT NOTE: =========================================================" >> $rcFile
    print ""                                                            >> $rcFile
    print "start_all()" 		 				>> $rcFile
    print "{" 						 		>> $rcFile
    print "   $CANDLEHOME/bin/CandleAgent start all" 			>> $rcFile
    print "}"  								>> $rcFile
    print "##########"  						>> $rcFile
    print "if [[ -f $CANDLEHOME/bin/CandleAgent ]]; then" 		>> $rcFile
    print "  start_all" 						>> $rcFile
    print "fi" 								>> $rcFile
    chmod 755 $rcFile

    typeset -i vline=0
    vline=$(sed -n '/^rcnfs:/p' /etc/inittab | wc -l)
    if [[ vline -gt 0 ]]
    then
      mkitab -i rcnfs "rcitm$filenum:2:once:$rcFile > /dev/console 2>&1" # IBM/Tivoli Monitoring Agents
    else
      vline=$(sed -n '/^rctcpip:/p' /etc/inittab | wc -l)
      if [[ vline -gt 0 ]]
      then
        mkitab -i rctcpip "rcitm$filenum:2:once:$rcFile > /dev/console 2>&1" # IBM/Tivoli Monitoring Agents
      else
        mkitab "rcitm$filenum:2:once:$rcFile > /dev/console 2>&1" # IBM/Tivoli Monitoring Agents
      fi
    fi

  else
    print "CANDLEHOME=$CANDLEHOME" 		>> $rcFile
    print "export CANDLEHOME" 			>> $rcFile
    cat $CANDLEHOME/bin/ITMAgents.sh 		>> $rcFile
    chmod 755 $rcFile
  fi
fi

# create start file and stop file.
create_start_stop_file

if [[ -s "$startFile" && -s "$stopFile" ]]; then
  delete_start_all
  exec 3< $startFile
  while read -u3 line ; do
    add_start_line
  done
  exec 3<&-

  if [[ "$os" != "ai" ]]; then
    delete_stop_all
    exec 3< $stopFile
    while read -u3 line ; do
      add_stop_line
    done
    exec 3<&-
  fi
else
  print "Start or Stop file is empty" >> $logFile
fi

######################################################################
#   END OF MAIN   ####################################################
######################################################################
